% Generate filterbank outputs using the filtering view of the STFT
clear
close all
clc

%% Analysis step

[x fs] = wavread('a075Leaf.wav');

% Subsample waveform (speeds things up)
x  = x(1:4:end);
fs = fs/4;
n  = 1:length(x);

nfft = 1024;  % Number of frequency bins

% Define filter
wlen = round(5*1e-3*fs);  % Broadband
win  = hamming(wlen);

% Compute filter bank
X = zeros(length(x),nfft/2);
for k=1:nfft/2
  
  % Modulated signal
  w0     = 2*pi*k/nfft;
  xmod   = x.*exp(-j*w0*n)';
  
  % Convolution with filter
  X(:,k) = filter(win,1,xmod);
  
  % Visualization
  if rem(k,10)==0
    hold off
    plot(abs(X(:,k)))
    axis tight; xlabel('time'); ylabel('Filter output');
    drawnow
    soundsc(abs(X(:,k)))
    pause
  end
end

%% Plot spectrogram
maxf = round(nfft * 4000/fs); % Max frequency to plot on spectrogram

figure
subplot(211)
sgram = log((abs(X(:,1:maxf)).^2))';
imagesc(flipud(sgram));
ylabel('Frequency')
subplot(212)
plot(x)
axis tight; xlabel('time'); ylabel('Amplitude')
