% Time-frequency resolution tradeoff (Quatieri fig 7.8)

clear
close all
clc

%% Generate chirp signal
t1   = 0;
t2   = 25e-3;
f1   = 1000;
f2   = 500;

fs   = 44100;
dt   = 1/fs;
t    = t1:dt:t2;
nfft = 2^14;

x = chirp(t,f1,t2,f2);

subplot(211)
plot(t,x, 'k');
axis tight 
xlabel('Time (s)')
ylabel('Chirp')
title('Temporal resolution');

%% Compute FFT at time 12.5ms with three different window lengths

wlen   = [5 10 20]*1e-3*fs;
ccolor = 'rgb';
for k=1:length(wlen)
  
  % Analysis window
  ix = round(length(t)/2 + (-wlen(k)/2:wlen(k)/2)); 
  x1 = x(ix);
  
  subplot(211)
  w = zeros(length(x),1);
  w(ix) = 1/3;
  w = w+k/3-1;
  hold on
  plot(t,w, 'Color', ccolor(k), 'LineWidth',2);
  
  % FFT
  fft1 = abs(fft(x1,nfft));
  subplot(212)
  freq = linspace(0,fs/20,nfft/20);
  plot(freq,fft1(1:round(nfft/20)), 'Color', ccolor(k));
  hold on
end
legend('5ms','10ms','20ms');
axis tight
xlabel('Frequency (Hz)')
ylabel('Amplitude')
title('Frequency resolution');
