% Estimate a signal from its STFT magnitude w/ Quatieri's LSE iteration
clc
clear
close all

nit = 50; % Number of iterations for reconstruction

[x,fs] = wavread('a075Leaf.wav');
x      = x';

% Compute spectrogram
wlen   = round(10e-3*fs);
wshft  = round(1e-3*fs);
win    = triang(wlen)';
nfft   = 1024;
stft_x = stft(x,win,wshft,nfft);

%% Re-estimate signal from STFT magnitude
stftm_hat = abs(stft_x);
mse       = [];
xhat      = 0.001*rand(size(lse(stftm_hat,win,wshft,nfft)));
xhat_old  = xhat*0;

for it=0:nit
  % Compute STFT of xhat and replace its magnitude with desired one
  stft_hat = stft(xhat,win,wshft,nfft);
  stft_hat = stft_hat(1:size(stftm_hat,1),:);
  stft_hat = stft_hat .* stftm_hat ./ abs(stft_hat);

  % Re-estimate signal
  xhat_old = xhat;
  xhat     = lse(stft_hat,win,wshft,nfft);
  mse      = [mse; sum((xhat_old-xhat).^2)];

  % Visualization
  subplot(311); plot(x);        axis tight
  subplot(312); plot(xhat);     axis tight; 
  subplot(313); plot(log(mse)); axis tight; drawnow
  soundsc(xhat,fs); pause(2*length(xhat)/fs)
  pause
end

fprintf('\n');

subplot(311); plot(xhat); hold on; plot(x,'r'); axis tight
subplot(312); specgram(x,   nfft,fs,win,wlen-wshft)
subplot(313); specgram(xhat,nfft,fs,win,wlen-wshft)

soundsc(x,fs)
pause(2*length(x)/fs)
soundsc(xhat,fs)

